
require 'parsedate'
require 'date'
require 'cgi'

module Fit


  # Copyright (c) 2002 Cunningham & Cunningham, Inc.
  # Released under the terms of the GNU General Public License version 2 or later.

  class Fixture 

    @@summary = {}
    @@right = @@wrong = @@ignores = @@exceptions = 0

    def Fixture.total_errors
      @@wrong + @@exceptions
    end

    def Fixture.summary
      @@summary
    end

    def summary
      @@summary
    end

    #// Traversal //////////////////////////

    def do_tables(tables)   
      summary["run date"] = Time.now
      until tables.nil?
        heading = tables.at(0,0,0)
        unless heading.nil?
          begin
            fixture = Fixture.find(heading.text())
            fixture.do_table(tables)
          rescue Exception => e
            Fixture.exception(heading, e)
          end
        end
        tables = tables.more
      end
    end

    def Fixture.find(name)
      klass = Fixture.get_class(name)
      raise "Fixture #{name} not found" unless klass
      raise "#{name} is not a fixture" unless klass < Fit::Fixture
      klass.new
    end

    # Try to load the named class. We first see if it's
    # already loaded by looking for the class name. If not,
    # we convert the class name to a file name (by changing
    # '::' to '/', then try to require that file. We then
    # look for the constant again. This means that
    # the class Example::Sqrt must be in the file 
    # Example/Sqrt

    def Fixture.get_class(name)
      klass = find_constant(name)
      unless klass
	file_name = name.gsub(/::/, '/')
	puts "Requiring #{name}"
	p $:
	begin
	  require file_name
	rescue Exception
	  raise "Couldn't find file #{file_name}"
	end
	klass = find_constant(name)
      end
      raise "Couldn't find class #{name}" unless klass
      klass
    end

    def Fixture.find_constant(name)
      parent = Object
      name.split(/::/).each do |next_class|
        parent = parent.const_get(next_class)
      end
      parent
    rescue Exception
      nil
    end

    def do_table(table) 
      do_rows(table.parts.more)
    end

    def do_rows(rows)
      until rows.nil?
        next_row = rows.more
        do_row(rows)
        rows = next_row
      end
    end

    def do_row(row) 
      do_cells(row.parts)
    end

    def do_cells(cells)
      i = 0
      until cells.nil?
        begin
          do_cell(cells, i)
        rescue Exception => e
          Fixture.exception(cells, e)
        end
        cells = cells.more
        i += 1
      end
    end

    def do_cell(cell, column_number)
      Fixture.ignore(cell)
    end



    # // Annotation ///////////////////////////////

    def Fixture.right(cell, update_count = true)
      cell.add_to_tag(" bgcolor=\"#cfffcf\"")
      @@right += 1 if update_count
    end

    def Fixture.wrong(cell, actual=nil, update_count = true)
      cell.add_to_tag(" bgcolor=\"#ffcfcf\"")
      if actual
        cell.add_to_body(label("expected") + 
                       "<hr>" +
                       escape(actual) + 
                       label("actual"))
      end
      @@wrong += 1 if update_count
    end

    def Fixture.ignore(cell)
      cell.add_to_tag(" bgcolor=\"#efefef\"")
      @@ignores += 1
    end

    def Fixture.exception(cell, e)
      backtrace = escape(e.backtrace.join("\n"))
      cell.add_to_body("<hr><font size=-2><pre>#{escape(e.message)}\n#{backtrace}</pre></font>");
      cell.add_to_tag(" bgcolor=\"#ffffcf\"");
      @@exceptions += 1
    end

    # // Utility //////////////////////////////////

    def Fixture.counts
      "#{@@right} right, " +
	"#{@@wrong} wrong, " +
	"#{@@ignores} ignored, " +
	"#{@@exceptions} exceptions"
    end

    def Fixture.label(string)
      " <font size=-1 color=#c08080><i>#{string}</i></font>"
    end

    def Fixture.gray(string)
      " <font color=#808080>#{string}</font>"
    end

    def Fixture.escape(string)
      string.gsub(/&/, '&amp;').gsub(/</, '&lt;')
    end

    #public static String escape (String string, char from, String to) {
    #    int i=-1;
    #    while ((i = string.indexOf(from, i+1)) >= 0) {
    #        if (i == 0) {
    #            string = to + string.substring(1);
    #          } else if (i == string.length()) {
    #              string = string.substring(0, i) + to;
    #            } else {
    #              string = string.substring(0, i) + to + string.substring(i+1);
    #            }
    #                 }
    #          return string;
    #        }

    def Fixture.camel(name)
      name.gsub(/\s*(\w)/) { $1.upcase }
    end

    def parse(string, klass)
      case klass
      when String
        string
      when Date
        date = ParseDate.parsedate(string)
        Date.new(*date[0,3])
      when Time
        date = ParseDate.parsedate(string)
        Time.new(*date[0,6])
      else
        begin
          klass(s)
        rescue Exception
          raise "Can't yet parse #{klass}"
        end
      end
    end

    def check(cell, type_adapter)
      text = cell.text()
      if text.empty?
        # skip cell
      elsif type_adapter.nil?
        Fixture.ignore(cell)
      elsif text == "error"
        begin
          result = type_adapter.invoke()
          Fixture.wrong(cell, type_adapter.to_s(result))
          #        } catch (IllegalAccessException e) {
          #                exception (cell, e);
        rescue Exception
          Fixture.right(cell)
        end
      else
        begin
          result = type_adapter.get()
          if type_adapter.equals(type_adapter.parse(text), result)
            Fixture.right(cell)
          else
            Fixture.wrong(cell, type_adapter.to_s(result))
          end
        rescue Exception => e
          Fixture.exception(cell, e)
        end
      end
    end

  end
end
